import type { CSSProperties } from "react";
import { useCallback, useEffect, useState } from "react";
import { ChevronRight, ChevronLeft } from "@carbon/icons-react";
import clsx from "clsx";
import useEmblaCarousel from "embla-carousel-react";
import { EmblaCarouselType } from "embla-carousel";
import { Lightbox } from "./Lightbox";
import { IImage } from "@interfaces/IImage";

interface SliderBaseProps {
  className?: string;
  title?: string;
  fullBleed?: boolean;
  id?: string;
  children: ({
    activeSlideIndex,
    setLightboxOpen,
  }: {
    activeSlideIndex: number;
    setLightboxOpen: (isOpen: boolean) => void;
  }) => React.ReactNode;
  loop?: boolean;
}

interface BaseSliderNoLightboxProps extends SliderBaseProps {
  withLightbox: true;
  items: IImage[];
}

interface BaseSliderWithLightboxProps extends SliderBaseProps {
  withLightbox?: false;
  items: unknown[];
}

export type BaseSliderProps =
  | BaseSliderNoLightboxProps
  | BaseSliderWithLightboxProps;

export function BaseSlider({
  className = "",
  title,
  fullBleed,
  id,
  children,
  loop,
  withLightbox,
  items,
}: BaseSliderProps) {
  const [emblaRef, emblaApi] = useEmblaCarousel({ containScroll: false, loop });
  const [prevBtnDisabled, setPrevBtnDisabled] = useState(true);
  const [nextBtnDisabled, setNextBtnDisabled] = useState(true);
  const [activeSlideIndex, setActiveSlideIndex] = useState(0);
  const [isLightboxOpen, setIsLightboxOpen] = useState(false);

  const onSelect = useCallback((emblaApi: EmblaCarouselType) => {
    setPrevBtnDisabled(!emblaApi.canScrollPrev());
    setNextBtnDisabled(!emblaApi.canScrollNext());
  }, []);

  useEffect(() => {
    if (!emblaApi) return;

    onSelect(emblaApi);
    emblaApi.on("reInit", onSelect).on("select", onSelect);
  }, [emblaApi, onSelect]);

  const scrollPrev = useCallback(() => {
    if (emblaApi) {
      setActiveSlideIndex((oldState) => (oldState + 1) % items.length);
      emblaApi.scrollPrev();
    }
  }, [emblaApi, setActiveSlideIndex, items]);

  const scrollNext = useCallback(() => {
    if (emblaApi) {
      setActiveSlideIndex((oldState) => (oldState + 1) % items.length);
      emblaApi.scrollNext();
    }
  }, [emblaApi, setActiveSlideIndex, items]);

  const fullBleedClasses: CSSProperties = fullBleed
    ? {
        width: "100vw",
        position: "relative",
        marginLeft: "-50vw",
        left: "50%",
      }
    : {};

  return (
    <>
      <div className={className} id={id}>
        {title && (
          <div className="container">
            <h3 className="fs-font-inter sm:fs-font-karla fs-text-3xl md:fs-text-5xl fs-leading-9 md:fs-leading-none fs-font-normal fs-text-grey-900 fs-mb-6 md:fs-mb-8">
              {title}
            </h3>
          </div>
        )}
        <div
          style={fullBleedClasses}
          className={clsx(
            "embla fs-grid fs-grid-cols-1",
            !fullBleed && "fs-overflow-hidden",
          )}
        >
          <div
            className="embla__viewport fs-col-start-1 fs-row-start-1 fs-overflow-hidden"
            ref={emblaRef}
          >
            <div className="embla__container fs-flex">
              {children({
                activeSlideIndex,
                setLightboxOpen: (isOpen) => {
                  setIsLightboxOpen(isOpen);
                },
              })}
            </div>
          </div>
          <div className="fs-col-start-1 fs-row-start-1 fs-size-full fs-flex fs-justify-between fs-items-center fs-px-4">
            <SliderNavigationButton
              direction="left"
              className="embla__prev"
              onClick={scrollPrev}
              disabled={prevBtnDisabled}
            />
            <SliderNavigationButton
              direction="right"
              className="embla__next"
              onClick={scrollNext}
              disabled={nextBtnDisabled}
            />
          </div>
        </div>
      </div>
      {withLightbox && (
        <Lightbox
          isOpen={isLightboxOpen}
          onClickNext={scrollNext}
          onClickPrev={scrollPrev}
          onClose={() => {
            setIsLightboxOpen(false);
          }}
        >
          <img
            className="fs-not-prose fs-size-full fs-my-0 fs-mx-auto fs-object-contain"
            src={items[activeSlideIndex].src}
            alt={items[activeSlideIndex].alt}
            width={items[activeSlideIndex].width}
            height={items[activeSlideIndex].height}
          />
        </Lightbox>
      )}
    </>
  );
}

interface SliderNavigationButtonProps {
  disabled?: boolean;
  direction: "left" | "right";
  onClick: (e: React.MouseEvent<HTMLButtonElement, MouseEvent>) => void;
  className?: string;
}

const SliderNavigationButton: React.FC<SliderNavigationButtonProps> = ({
  disabled,
  direction,
  onClick,
  className,
}: SliderNavigationButtonProps) => {
  return (
    <button
      onClick={onClick}
      className={clsx(
        "fs-z-10 fs-cursor-pointer fs-size-12 fs-flex fs-justify-center fs-items-center fs-bg-grey-900 fs-shadow-sm fs-rounded-3xl",
        disabled
          ? "fs-invisible"
          : "fs-opacity-50 fs-transition-opacity fs-duration-500 fs-ease-in-out hover:fs-opacity-100",
        className,
      )}
      title={direction === "left" ? "Scroll left" : "Scroll right"}
    >
      {direction === "left" && (
        <ChevronLeft size={32} className="fs-fill-white" />
      )}
      {direction === "right" && (
        <ChevronRight size={32} className="fs-fill-white" />
      )}
    </button>
  );
};
