import { useState } from "react";
import { Earth } from "@carbon/icons-react";

export interface LanguageSelectorProps {
  // Mobile menu overlay when Hamburger button is clicked
  isMobileOpen?: boolean;
  // Show options horizontally instead of "dropdown on hover"
  horizontal?: boolean;
  languages: Array<string>;
  selectedLanguage?: string;
  onChangeLanguage?: (newLanguage: string) => void;
  faded?: boolean;
  onHover?: (value: boolean) => void;
  // Use light primary colors (e.g. for use with dark backgrounds)
  light?: boolean;
}

// TODO: Horizontal Mode is not working – Language Selector is currently not being used in Topbar
export function LanguageSelector({
  horizontal,
  selectedLanguage,
  languages,
  onChangeLanguage,
  onHover,
  isMobileOpen,
  faded,
  light,
}: LanguageSelectorProps) {
  // Selector is open – options are visible
  const [isOpen, setIsOpen] = useState(false);

  const getIconColor = (
    isMobileOpen?: boolean,
    faded?: boolean,
    light?: boolean,
  ) => {
    if (isMobileOpen) return "fs-text-white";
    if (!faded) {
      return light
        ? "fs-text-grey-900 sm:fs-text-white lg:fs-text-grey-900"
        : "fs-text-grey-900";
    }
    if (faded) {
      return "fs-text-grey-400 hover:fs-text-grey-900";
    }
  };

  const getLocaleColor = (
    isMobileOpen?: boolean,
    light?: boolean,
    isActive?: boolean,
    horizontal?: boolean,
  ) => {
    // Mobile overlay (always blue/white)
    if (isMobileOpen) return "fs-text-white";

    // Mobile
    if (horizontal) {
      if (light)
        return isActive
          ? "fs-text-grey-900 sm:fs-text-white"
          : "fs-text-grey-400";
      if (!light) return isActive ? "fs-text-grey-900" : "fs-text-grey-400";
    } else {
      // Desktop
      if (light)
        return isActive
          ? "fs-text-grey-900 sm:fs-text-white"
          : "fs-text-grey-400";
      if (!light) return isActive ? "fs-text-white" : "fs-text-grey-400";
    }
  };

  const handleLanguageClick = (language: string) => {
    if (language === selectedLanguage) return;
    setIsOpen(false);
    onChangeLanguage && onChangeLanguage(language);
  };

  const iconColor = getIconColor(isMobileOpen, faded, light);

  return languages ? (
    <>
      {horizontal ? (
        <div className="fs-relative lg:fs-hidden fs-flex fs-items-center">
          <button
            onClick={() => setIsOpen(!isOpen)}
            className={`fs-order-2 fs-z-10 ${iconColor || ""}`}
          >
            <Earth size={24} />
          </button>
          <div
            className={`fs-absolute fs-right-0 fs-opacity-0 ${
              isOpen ? "!fs-right-full fs-opacity-100" : ""
            } fs-order-1 fs-flex fs-px-4 fs-font-plex fs-text-sm fs-leading-5 fs-font-semibold fs-transition-all fs-ease-in fs-duration-50`}
          >
            {languages &&
              languages.map((language) => {
                const isActive = language === selectedLanguage;
                const classes = getLocaleColor(
                  isMobileOpen,
                  light,
                  isActive,
                  horizontal,
                );
                return (
                  <div key={language} className="fs-px-5">
                    <button
                      onClick={() => handleLanguageClick(language)}
                      className={classes}
                    >
                      {language.toUpperCase()}
                    </button>
                  </div>
                );
              })}
          </div>
        </div>
      ) : (
        <div className="fs-h-full">
          <div
            onMouseEnter={() => {
              onHover && onHover(true);
            }}
            onMouseLeave={() => {
              onHover && onHover(false);
            }}
            className={`fs-group fs-relative fs-flex fs-items-center fs-cursor-pointer fs-h-full fs-transition-all fs-ease-in fs-duration-100 ${
              iconColor || ""
            }`}
          >
            <Earth size={20} />
            <div className="group-hover:fs-block fs-hidden fs-absolute fs-left-1/2 -fs-translate-x-2/4 fs-top-full fs-bg-grey-900 fs-font-plex fs-text-sm fs-leading-5 fs-font-semibold">
              {languages &&
                languages.map((language, index) => {
                  const isActive = language === selectedLanguage;
                  const colorClasses = getLocaleColor(
                    isMobileOpen,
                    light,
                    isActive,
                    horizontal,
                  );

                  return (
                    <div
                      key={language}
                      className={`fs-px-3.5 ${
                        index === 0
                          ? "fs-pt-8 fs-pb-4"
                          : index === languages.length - 1
                            ? "fs-pt-4 fs-pb-8"
                            : "fs-py-4"
                      }`}
                    >
                      <button
                        onClick={() => handleLanguageClick(language)}
                        className={colorClasses}
                      >
                        {language.toUpperCase()}
                      </button>
                    </div>
                  );
                })}
            </div>
          </div>
        </div>
      )}
    </>
  ) : null;
}
