import React, {
  useState,
  useEffect,
  Children,
  isValidElement,
  Fragment,
} from "react";
import { CommonUtils } from "@utils/CommonUtils";
import { ITabItem } from "@interfaces/ITab";
import clsx from "clsx";

type BaseProps = {
  className?: string;
};

type ChildrenProps = {
  children?: React.ReactNode;
  entries?: never;
};

type EntriesProps = {
  children?: never;
  entries?: ITabItem[];
};

export type TabsProps = BaseProps & (ChildrenProps | EntriesProps);

interface ITabContent {
  id: string;
  label: string;
  content: string;
}

export function Tabs({ children, entries, className = "" }: TabsProps) {
  const [activeTab, setActiveTab] = useState<string | null>(null);
  const [tabData, setTabData] = useState<ITabContent[]>([]);

  useEffect(() => {
    if (children) {
      const data: ITabContent[] = [];
      Children.forEach(children, (child) => {
        if (!isValidElement(child)) return;
        data.push({
          id: CommonUtils.generateID(),
          label: child.props.label,
          content: child.props.children,
        });
      });
      setTabData(data);
      setActiveTab(data[0].id);
    }
  }, [children]);

  useEffect(() => {
    if (entries) {
      const data: ITabContent[] = entries.map((entry) => {
        return {
          id: CommonUtils.generateID(),
          label: entry.label,
          content: entry.content,
        };
      });
      setTabData(data);
      setActiveTab(data[0].id);
    }
  }, [entries]);

  return (
    <div className={clsx("fs-w-full", className)}>
      <ol className="fs-not-prose fs-flex fs-overflow-x-auto fs-overflow-y-hidden">
        {tabData.map((tab) => (
          <TabPanel
            key={tab.id}
            label={tab.label}
            active={tab.id === activeTab}
            onClick={() => setActiveTab(tab.id)}
          />
        ))}
      </ol>
      <div className="fs-py-4 xl:fs-py-6">
        {tabData.map((tab) => {
          if (activeTab === tab.id) {
            return <TabContent key={tab.id} content={tab.content} />;
          }
        })}
      </div>
    </div>
  );
}

interface TabPanelProps {
  label: string;
  active: boolean;
  onClick: () => void;
}

function TabPanel({ label, active, onClick }: TabPanelProps) {
  const beforeClasses =
    "before:fs-block before:fs-content-[attr(data-label)] before:fs-font-semibold before:fs-h-0 before:fs-overflow-hidden before:fs-invisible";
  const afterClasses =
    "after:fs-content-[''] after:fs-absolute after:fs-left-0 after:-fs-bottom-0.5 after:fs-bg-grey-transparent hover:after:fs-bg-grey-500 after:fs-scale-x-0 hover:after:fs-scale-x-100 after:fs-transition-all after:fs-duration-300 after:fs-ease-in-out after:fs-origin-left after:fs-w-full after:fs-h-0.5";
  return (
    <li
      className={clsx(
        "fs-group fs-relative fs-inline-block fs-text-[0px] fs-px-4 xl:fs-px-5 fs-py-2 xl:fs-py-3 fs-cursor-pointer fs-select-none fs-border-b-2 fs-border-transparent",
        afterClasses,
        active &&
          "after:fs-scale-x-100 after:fs-bg-grey-900 hover:after:fs-bg-grey-900",
      )}
      onClick={onClick}
    >
      <span
        className={clsx(
          "xl:group-hover:fs-font-semibold fs-inline-block fs-whitespace-nowrap fs-font-plex fs-text-center fs-text-base xl:fs-text-lg fs-leading-6",
          beforeClasses,
          active
            ? "fs-font-semibold fs-text-grey-900 fs-text-lg"
            : "fs-font-medium xl:fs-font-normal fs-text-grey-500",
        )}
        data-label={label}
      >
        {label}
      </span>
    </li>
  );
}

interface TabContentProps {
  content: string;
  prose?: boolean;
}

function TabContent({ content }: TabContentProps) {
  return <div dangerouslySetInnerHTML={{ __html: content }}></div>;
}
interface TabProps {
  children: React.ReactNode;
  label: string;
}

export const Tab: React.FC<TabProps> = ({ children }) => {
  return <Fragment>{children}</Fragment>;
};
